@@ -77,7 +77,7 @@
 #' \item{init.model:}{Right-hand side of the formula corresponding to the
 #'       initial model.}
 #' \item{final.model:}{Right-hand side of the formula corresponding to the
-#'       final model.}
+#'       final model after forward selection.}
 #' \item{family:}{Type of model fitted.}
 #' \item{params:}{List of parameters used.}
 #' \item{iter1:}{Summary statistics for all variables at the first iteration.}
@@ -97,7 +97,8 @@
 #'
 #' # close the parallel cluster
 #' stopImplicitCluster()
-#' @seealso \code{\link{nested.forward.selection}}
+#'
+#' @seealso \code{\link{nested.forward.selection}} and \code{\link{summary.fs}}.
 #' @keywords multivariate
 #' @importFrom foreach foreach %dopar%
 #' @importFrom stats coefficients glm predict t.test update wilcox.test
@@ -364,12 +365,16 @@ forward.selection <- function(x, y, init.model, family,
 #'
 #' # close the parallel cluster
 #' stopImplicitCluster()
-#' @seealso \code{\link{forward.selection}}
+#'
+#' @seealso
+#' \code{\link{forward.selection}}, \code{\link{summary.nestfs}} and
+#' \code{\link{nested.performance}}.
 #' @keywords multivariate
 #' @export
 nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
 
   res <- list()
+  y <- validate.outcome(y)
   family <- validate.family(family, y)
   folds <- validate.folds(folds, x)
   verbose <- isTRUE(list(...)$verbose) || is.null(list(...)$verbose)
@@ -386,7 +391,10 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
     x.train <- x[train.idx, ]; y.train <- y[train.idx]
 
     fs <- forward.selection(x.train, y.train, init.model, family, ...)
-    model <- glm.inner(x[, fs$fs$vars], y, test.idx, family)
+
+    ## fit the final model on the training set and evaluate it on the test set
+    model <- validate.init.model(fs$final.model)
+    model <- glm.inner(x, y, model, test.idx, family)
     stopifnot(all.equal(model$obs, y[test.idx]))
 
     ## extract the model coefficients and report them in the forward selection
@@ -422,17 +430,20 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
 #'
 #' @param x Dataframe of predictors.
 #' @template args-outcome
+#' @param model Either a formula or a vector of names for the set of variables
+#'        that define the model to be fitted.
 #' @template args-family
 #' @template args-folds
 #' @param store.glm Whether the object produced by \code{glm} should be
 #'        stored (default: \code{FALSE}).
 #'
 #' @return
-#' A list of length equal to \code{length(folds)}, where each entry contains
-#' the following fields:
+#' An object of class \code{nestglm} of length equal to \code{length(folds)},
+#' where each entry contains the following fields:
 #' \describe{
 #' \item{summary:}{Summary of the coefficients of the model fitted on the
 #'       training observations.}
+#' \item{family:}{Type of model fitted.}
 #' \item{fit:}{Predicted values for the withdrawn observations.}
 #' \item{obs:}{Observed values for the withdrawn observations.}
 #' \item{test.llk:}{Test log-likelihood.}
@@ -447,24 +458,34 @@ nested.forward.selection <- function(x, y, init.model, family, folds, ...) {
 #'
 #' data(diabetes)
 #' folds <- create.folds(10, nrow(X.diab), seed=1)
-#' base.res <- nested.glm(X.diab[, c("age", "sex", "bmi", "tc",
-#'                                   "ldl", "hdl", "ltg", "glu")],
-#'                        Y.diab, gaussian(), folds)
+#' base.res <- nested.glm(X.diab, Y.diab, c("age", "sex", "bmi", "map"),
+#'                        gaussian(), folds)
 #'
 #' # close the parallel cluster
 #' stopImplicitCluster()
+#'
+#' @seealso \code{\link{nested.performance}}.
 #' @keywords multivariate
 #' @export
-nested.glm <- function(x, y, family, folds, store.glm=FALSE) {
+nested.glm <- function(x, y, model, family, folds, store.glm=FALSE) {
 
   ## argument checks
   if (nrow(x) != length(y))
     stop("Mismatched dimensions.")
   y <- validate.outcome(y)
+  model <- validate.init.model(model)
   family <- validate.family(family, y)
   folds <- validate.folds(folds, x)
 
-  res <- lapply(folds, function(z) glm.inner(x, y, z, family, store.glm))
+  ## check that all variables exist in the dataframe of predictors
+  vars <- setdiff(all.vars(model), "nestfs_y_")
+  var.match <- match(vars, colnames(x))
+  if (anyNA(var.match))
+    stop("'", paste(vars[is.na(var.match)], collapse="', '"),
+         "' not present in x.")
+
+  res <- lapply(folds, function(z) glm.inner(x, y, model, z, family, store.glm))
+  class(res) <- c("nestglm")
   return(res)
 }
 
@@ -474,9 +495,9 @@ nested.glm <- function(x, y, family, folds, store.glm=FALSE) {
 #' Fit a model using all predictors provided on the training observations, then
 #' test it on the withdrawn observations.
 #'
-#' @param x Dataframe of predictors containing all and only the variables to be
-#'        used in the model.
+#' @param x Dataframe of predictors.
 #' @template args-outcome
+#' @param model Formula for the model to be fitted.
 #' @param idx.test Indices of observations to withdraw.
 #' @template args-family
 #' @param store.glm Whether the object produced by \code{glm} should be
@@ -487,14 +508,13 @@ nested.glm <- function(x, y, family, folds, store.glm=FALSE) {
 #'
 #' @importFrom stats as.formula glm predict
 #' @noRd
-glm.inner <- function(x, y, idx.test, family, store.glm=FALSE) {
+glm.inner <- function(x, y, model, idx.test, family, store.glm=FALSE) {
   idx.train <- setdiff(1:nrow(x), idx.test)
-  model <- paste("y ~", paste(colnames(x), collapse=" + "))
-  regr <- glm(as.formula(model), data=x, family=family, subset=idx.train)
+  regr <- glm(model, data=cbind(nestfs_y_=y, x)[idx.train, ], family=family)
   y.pred <- predict(regr, newdata=x[idx.test, ], type="response")
   y.test <- y[idx.test]
   loglik <- loglikelihood(family, y.test, y.pred, summary(regr)$dispersion)
-  res <- list(summary=coefficients(summary(regr)),
+  res <- list(summary=coefficients(summary(regr)), family=family$family,
               fit=y.pred, obs=y.test, test.llk=loglik, test.idx=idx.test)
   if (store.glm) res$regr <- regr
   return(res)
@@ -608,7 +628,7 @@ validate.family <- function(family, y) {
   if (family$family == "binomial") {
     if (length(table(y)) != 2)
       stop("y must contain two classes with family=binomial().", call.=FALSE)
-    if (any(y < 0 | y > 1))
+    if (!is.factor(y) && any(y < 0 | y > 1))
       stop("y must contain 0-1 values with family=binomial().", call.=FALSE)
   }
 
@@ -648,7 +668,7 @@ validate.choose.from <- function(choose.from, x) {
     else
       stop("choose.from should be an integer or character vector.", call.=FALSE)
   }
-  return(choose.from)
+  return(as.integer(choose.from))
 }
 
 #' Validate the folds argument
